@@ -49,16 +49,16 @@ import wicd.wpath as wpath
 import wicd.misc as misc
 import wicd.gui as gui
 
+ICON_AVAIL = True
+USE_EGG = False
 # Import egg.trayicon if we're using an older gtk version
-if not (gtk.gtk_version[0] >= 2 and gtk.gtk_version[1] >= 10):
+if not hasattr(gtk, "StatusIcon"):
     try:
         import egg.trayicon
         USE_EGG = True
     except ImportError:
-        print 'Unable to load wicd-client.py: Missing egg.trayicon module.'
-        sys.exit(1)
-else:
-    USE_EGG = False
+        print 'Unable to load tray icon: Missing egg.trayicon module.'
+        ICON_AVAIL = False
 
 if getattr(dbus, 'version', (0, 0, 0)) < (0, 80, 0):
     import dbus.glib
@@ -88,7 +88,7 @@ language['connecting'] = _('Connecting')
 language['wired'] = _('Wired Network')
 
 
-class TrayIcon:
+class TrayIcon(object):
     """ Base Tray Icon class.
     
     Base Class for implementing a tray icon to display network status.
@@ -101,8 +101,14 @@ class TrayIcon:
             self.tr = self.StatusTrayIconGUI(use_tray)
         self.icon_info = self.TrayConnectionInfo(self.tr, use_tray, animate)
         
+    def is_embedded(self):
+        if USE_EGG:
+            raise NotImplementedError
+        else:
+            return self.tr.is_embedded()
+        
 
-    class TrayConnectionInfo:
+    class TrayConnectionInfo(object):
         """ Class for updating the tray icon status. """
         def __init__(self, tr, use_tray=True, animate=True):
             """ Initialize variables needed for the icon status methods. """
@@ -290,7 +296,7 @@ class TrayIcon:
             return (active, max_gain, last_bytes)
 
 
-    class TrayIconGUI:
+    class TrayIconGUI(object):
         """ Base Tray Icon UI class.
         
         Implements methods and variables used by both egg/StatusIcon
@@ -346,7 +352,8 @@ class TrayIcon:
             """ Opens the About Dialog. """
             dialog = gtk.AboutDialog()
             dialog.set_name('wicd tray icon')
-            dialog.set_version('1.5')
+            # VERSIONNUMBER
+            dialog.set_version('1.5.6')
             dialog.set_comments('An icon that shows your network connectivity')
             dialog.set_website('http://wicd.net')
             dialog.run()
@@ -370,96 +377,97 @@ class TrayIcon:
                 self.gui_win.exit()
                 return True
         
-
-    class EggTrayIconGUI(TrayIconGUI):
-        """ Tray Icon for gtk < 2.10.
-        
-        Uses the deprecated egg.trayicon module to implement the tray icon.
-        Since it relies on a deprecated module, this class is only used
-        for machines running versions of GTK < 2.10.
-        
-        """
-        def __init__(self, use_tray=True):
-            """Initializes the tray icon"""
-            TrayIcon.TrayIconGUI.__init__(self, use_tray)
-            self.use_tray = use_tray
-            if not use_tray: 
-                self.toggle_wicd_gui()
-                return
-
-            self.tooltip = gtk.Tooltips()
-            self.eb = gtk.EventBox()
-            self.tray = egg.trayicon.TrayIcon("WicdTrayIcon")
-            self.pic = gtk.Image()
-            self.tooltip.set_tip(self.eb, "Initializing wicd...")
-            self.pic.set_from_file("images/no-signal.png")
-
-            self.eb.connect('button_press_event', self.tray_clicked)
-            self.eb.add(self.pic)
-            self.tray.add(self.eb)
-            self.tray.show_all()
-
-        def tray_clicked(self, widget, event):
-            """ Handles tray mouse click events. """
-            if event.button == 1:
-                self.toggle_wicd_gui()
-            elif event.button == 3:
-                self.menu.popup(None, None, None, event.button, event.time)
-
-        def set_from_file(self, val=None):
-            """ Calls set_from_file on the gtk.Image for the tray icon. """
-            if not self.use_tray: return
-            self.pic.set_from_file(val)
-
-        def set_tooltip(self, val):
-            """ Set the tooltip for this tray icon.
+    if USE_EGG:
+        class EggTrayIconGUI(TrayIconGUI):
+            """ Tray Icon for gtk < 2.10.
+            
+            Uses the deprecated egg.trayicon module to implement the tray icon.
+            Since it relies on a deprecated module, this class is only used
+            for machines running versions of GTK < 2.10.
             
-            Sets the tooltip for the gtk.ToolTips associated with this
-            tray icon.
-
             """
-            if not self.use_tray: return
-            self.tooltip.set_tip(self.eb, val)
+            def __init__(self, use_tray=True):
+                """Initializes the tray icon"""
+                TrayIcon.TrayIconGUI.__init__(self, use_tray)
+                self.use_tray = use_tray
+                if not use_tray: 
+                    self.toggle_wicd_gui()
+                    return
+    
+                self.tooltip = gtk.Tooltips()
+                self.eb = gtk.EventBox()
+                self.tray = egg.trayicon.TrayIcon("WicdTrayIcon")
+                self.pic = gtk.Image()
+                self.tooltip.set_tip(self.eb, "Initializing wicd...")
+                self.pic.set_from_file("images/no-signal.png")
+    
+                self.eb.connect('button_press_event', self.tray_clicked)
+                self.eb.add(self.pic)
+                self.tray.add(self.eb)
+                self.tray.show_all()
+    
+            def tray_clicked(self, widget, event):
+                """ Handles tray mouse click events. """
+                if event.button == 1:
+                    self.toggle_wicd_gui()
+                elif event.button == 3:
+                    self.menu.popup(None, None, None, event.button, event.time)
+    
+            def set_from_file(self, val=None):
+                """ Calls set_from_file on the gtk.Image for the tray icon. """
+                if not self.use_tray: return
+                self.pic.set_from_file(val)
+    
+            def set_tooltip(self, val):
+                """ Set the tooltip for this tray icon.
+                
+                Sets the tooltip for the gtk.ToolTips associated with this
+                tray icon.
+    
+                """
+                if not self.use_tray: return
+                self.tooltip.set_tip(self.eb, val)
 
-
-    class StatusTrayIconGUI(gtk.StatusIcon, TrayIconGUI):
-        """ Class for creating the wicd tray icon on gtk > 2.10.
-        
-        Uses gtk.StatusIcon to implement a tray icon.
-        
-        """
-        def __init__(self, use_tray=True):
-            TrayIcon.TrayIconGUI.__init__(self, use_tray)
-            self.use_tray = use_tray
-            if not use_tray: 
-                self.toggle_wicd_gui()
-                return
-
-            gtk.StatusIcon.__init__(self)
-
-            self.current_icon_path = ''
-            self.set_visible(True)
-            self.connect('activate', self.on_activate)
-            self.connect('popup-menu', self.on_popup_menu)
-            self.set_from_file(wpath.images + "no-signal.png")
-            self.set_tooltip("Initializing wicd...")
-
-        def on_popup_menu(self, status, button, timestamp):
-            """ Opens the right click menu for the tray icon. """
-            self.menu.popup(None, None, None, button, timestamp)
-
-        def set_from_file(self, path = None):
-            """ Sets a new tray icon picture. """
-            if not self.use_tray: return
-            if path != self.current_icon_path:
-                self.current_icon_path = path
-                gtk.StatusIcon.set_from_file(self, path)
+    if hasattr(gtk, "StatusIcon"):
+        class StatusTrayIconGUI(gtk.StatusIcon, TrayIconGUI):
+            """ Class for creating the wicd tray icon on gtk > 2.10.
+            
+            Uses gtk.StatusIcon to implement a tray icon.
+            
+            """
+            def __init__(self, use_tray=True):
+                TrayIcon.TrayIconGUI.__init__(self, use_tray)
+                self.use_tray = use_tray
+                if not use_tray: 
+                    self.toggle_wicd_gui()
+                    return
+    
+                gtk.StatusIcon.__init__(self)
+    
+                self.current_icon_path = ''
+                self.set_visible(True)
+                self.connect('activate', self.on_activate)
+                self.connect('popup-menu', self.on_popup_menu)
+                self.set_from_file(wpath.images + "no-signal.png")
+                self.set_tooltip("Initializing wicd...")
+    
+            def on_popup_menu(self, status, button, timestamp):
+                """ Opens the right click menu for the tray icon. """
+                self.menu.popup(None, None, None, button, timestamp)
+    
+            def set_from_file(self, path = None):
+                """ Sets a new tray icon picture. """
+                if not self.use_tray: return
+                if path != self.current_icon_path:
+                    self.current_icon_path = path
+                    gtk.StatusIcon.set_from_file(self, path)
 
 
 def usage():
+    # VERSIONNUMBER
     """ Print usage information. """
     print """
-wicd 1.50
+wicd 1.5.6
 wireless (and wired) connection daemon front-end.
 
 Arguments:
@@ -527,7 +535,7 @@ def main(argv):
     print 'Loading...'
     connect_to_dbus()
 
-    if not use_tray:
+    if not use_tray or not ICON_AVAIL:
         the_gui = gui.appGui()
         the_gui.standalone = True
         mainloop = gobject.MainLoop()
