@@ -55,10 +55,11 @@ else:
 import wicd.wpath as wpath
 import wicd.networking as networking
 import wicd.misc as misc
+from wicd.misc import noneToBlankString
 
 if __name__ == '__main__':
     wpath.chdir(__file__)
-    
+
 misc.RenameProcess("wicd")
 
 logging_enabled = True
@@ -66,8 +67,16 @@ logging_enabled = True
 class LogWriter:
     """ A class to provide timestamped logging. """
     def __init__(self):
+        # make the log directory
         if not os.path.exists(wpath.log + 'wicd.log'):
-            os.system("mkdir -p %s" % wpath.log)
+            try:
+                os.makedirs(wpath.log)
+            except OSError, e:
+                # ignore "file exists" errors
+                # it's possible the directory is already made
+                if not e.errno == 17:
+                    raise
+            os.chmod(wpath.log, 755)
         self.file = open(wpath.log + 'wicd.log', 'w')
         try:
             os.chmod(wpath.log + 'wicd.log', 0600)
@@ -77,6 +86,9 @@ class LogWriter:
 
     def __del__(self):
         self.file.close()
+        
+    def flush(self):
+        self.file.flush()
 
     def write(self, data):
         """ Writes the data to the log with a timestamp.
@@ -97,7 +109,7 @@ class LogWriter:
 
         """
         global logging_enabled
-        data = data.decode('utf-8').encode('utf-8')
+        data = misc.to_unicode(data)
         if len(data) <= 0: return
         if logging_enabled:
             if self.eol:
@@ -109,7 +121,7 @@ class LogWriter:
                 data = data[:-1]
 
             self.file.write(
-                    data.replace('\n', '\n' + self.get_time() + ' :: '))
+                data.replace('\n', '\n' + self.get_time() + ' :: '))
             if self.eol: self.file.write('\n')
             self.file.flush()
 
@@ -185,14 +197,15 @@ class ConnectionWizard(dbus.service.Object):
     @dbus.service.method('org.wicd.daemon')
     def Hello(self):
         """ Returns the version number. 
-        
+
         This number is major-minor-micro. Major is only incremented if minor
         reaches > 9. Minor is incremented if changes that break core stucture
         are implemented. Micro is for everything else, and micro may be
         anything >= 0. This number is effective starting wicd v1.2.0.
-        
+
         """
-        version = '1.5.2'
+        # VERSIONNUMBER
+        version = '1.5.6'
         print 'returned version number', version
         return version
 
@@ -200,8 +213,8 @@ class ConnectionWizard(dbus.service.Object):
     def SetWiredInterface(self, interface):
         """ Sets the wired interface for the daemon to use. """
         print "setting wired interface %s" % (str(interface))
-        self.wired.wired_interface = interface
-        self.wifi.wired_interface = interface
+        self.wired.wired_interface = noneToBlankString(interface)
+        self.wifi.wired_interface = noneToBlankString(interface)
         config = ConfigParser.ConfigParser()
         config.read(self.app_conf)
         config.set("Settings","wired_interface", interface)
@@ -211,8 +224,8 @@ class ConnectionWizard(dbus.service.Object):
     def SetWirelessInterface(self, interface):
         """ Sets the wireless interface the daemon will use. """
         print "setting wireless interface %s" % (str(interface))
-        self.wifi.wireless_interface = interface
-        self.wired.wireless_interface = interface
+        self.wifi.wireless_interface = noneToBlankString(interface)
+        self.wired.wireless_interface = noneToBlankString(interface)
         config = ConfigParser.ConfigParser()
         config.read(self.app_conf)
         config.set("Settings","wireless_interface", interface)
@@ -339,7 +352,7 @@ class ConnectionWizard(dbus.service.Object):
             return (signal + " dBm")
         else:
             return (signal + "%")
-    
+
     @dbus.service.method('org.wicd.daemon')
     def SetSuspend(self, val):
         """ Toggles whether or not monitoring connection status is suspended """
@@ -355,7 +368,7 @@ class ConnectionWizard(dbus.service.Object):
     @dbus.service.method('org.wicd.daemon')
     def AutoConnect(self, fresh):
         """ Attempts to autoconnect to a wired or wireless network.
-        
+
         Autoconnect will first try to connect to a wired network, if that 
         fails it tries a wireless connection.
 
@@ -366,14 +379,14 @@ class ConnectionWizard(dbus.service.Object):
             self._wired_autoconnect()
         else:
             self._wireless_autoconnect()
-    
+
     def _wired_autoconnect(self):
         """ Attempts to autoconnect to a wired network. """
         if self.GetWiredAutoConnectMethod() == 2 and \
            not self.GetNeedWiredProfileChooser():
             self.LaunchChooser()
             return
-        
+
         # Default Profile.
         elif self.GetWiredAutoConnectMethod() == 1:
             network = self.GetDefaultWiredNetwork()
@@ -438,16 +451,16 @@ class ConnectionWizard(dbus.service.Object):
         """ Returns the value of self.auto_reconnect. See SetAutoReconnect. """
         do = bool(self.auto_reconnect)
         return self.__printReturn('returning automatically reconnect when ' \
-                                   + 'connection drops', do)
+                                  + 'connection drops', do)
 
     @dbus.service.method('org.wicd.daemon')
     def SetAutoReconnect(self, value):
         """ Sets the value of self.auto_reconnect.
-        
+
         If True, wicd will try to reconnect as soon as it detects that
         an internet connection is lost.  If False, it will do nothing,
         and wait for the user to initiate reconnection.
-        
+
         """
         print 'setting automatically reconnect when connection drops'
         config = ConfigParser.ConfigParser()
@@ -469,7 +482,7 @@ class ConnectionWizard(dbus.service.Object):
             return True
         else:
             return False
-    
+
     @dbus.service.method('org.wicd.daemon')
     def CancelConnect(self):
         """ Cancels the wireless connection attempt """
@@ -479,12 +492,12 @@ class ConnectionWizard(dbus.service.Object):
         if self.wired.connecting_thread:
             self.wired.connecting_thread.should_die = True
         misc.Run("killall dhclient dhclient3 wpa_supplicant")
-    
+
     @dbus.service.method('org.wicd.daemon')
     def GetCurrentInterface(self):
         """ Returns the active interface """
         return self.current_interface
-    
+
     @dbus.service.method('org.wicd.daemon')    
     def SetCurrentInterface(self, iface):
         """ Sets the current active interface """
@@ -493,15 +506,15 @@ class ConnectionWizard(dbus.service.Object):
     @dbus.service.method('org.wicd.daemon')
     def SetNeedWiredProfileChooser(self, val):
         """ Sets the need_wired_profile_chooser variable.
-        
+
         If set to True, that alerts the wicd frontend to display the chooser,
         if False the frontend will do nothing.  This function is only needed
         when the frontend starts up, to determine if the chooser was requested
         before the frontend was launched.
-        
+
         """
         self.need_profile_chooser = misc.to_bool(val)
-        
+
     @dbus.service.method('org.wicd.daemon')
     def ShouldAutoReconnect(self):
         """ Returns True if it's the right time to try autoreconnecting. """
@@ -519,14 +532,14 @@ class ConnectionWizard(dbus.service.Object):
     @dbus.service.method('org.wicd.daemon')
     def SetForcedDisconnect(self, value):
         """ Sets the forced_disconnect status.
-        
+
         Set to True when a user manually disconnects or cancels a connection.
         It gets set to False as soon as the connection process is manually
         started.
-        
+
         """
         self.forced_disconnect = bool(value)
-    
+
     @dbus.service.method('org.wicd.daemon')
     def GetGUIOpen(self):
         """ Returns the value of gui_open.
@@ -535,32 +548,32 @@ class ConnectionWizard(dbus.service.Object):
         of the state of the wicd GUI.  If the GUI is open, wicd will not
         try to automatically reconnect to networks, as this behavior can
         be annoying for the user while trying to use the GUI.
-        
+
         NOTE: It's possible for this to become out of sync, particularly if
         the wicd.py is not exited properly while the GUI is open.  We should
         probably implement some kind of pid system to do it properly.
-        
+
         ANOTHER NOTE: This isn't implemented yet!
-        
+
         """
         return bool(self.gui_open)
-    
+
     @dbus.service.method('org.wicd.daemon')
     def SetGUIOpen(self, val):
         """ Sets the value of gui_open. """
         self.gui_open = bool(val)
-        
+
     @dbus.service.method('org.wicd.daemon')
     def SetConnectionStatus(self, state, info):
         """ Sets the connection status.
-        
+
         Keyword arguments:
         state - An int representing the state of the connection as defined
         in misc.py.
-        
+
         info - a list of strings containing data about the connection state.  
         The contents of this list are dependent on the connection state.
-        
+
         state - info contents:
         NOT_CONNECTED - info[0] = ""
         CONNECTING - info[0] = "wired" or "wireless"
@@ -571,12 +584,12 @@ class ConnectionWizard(dbus.service.Object):
                    info[2] = signal strength
                    info[3] = internal networkid
         SUSPENDED - info[0] = ""
-                
-        
+
+
         """
         self.connection_state = state
         self.connection_info = info
-    
+
     @dbus.service.method('org.wicd.daemon', out_signature='(uas)')
     def GetConnectionStatus(self):
         return [self.connection_state, self.connection_info]
@@ -584,17 +597,17 @@ class ConnectionWizard(dbus.service.Object):
     @dbus.service.method('org.wicd.daemon')
     def GetNeedWiredProfileChooser(self):
         """ Returns need_profile_chooser.
-        
+
         Returns a boolean specifying if the wired profile chooser needs to
         be launched.
-        
+
         """
         return bool(self.need_profile_chooser)
 
     @dbus.service.method('org.wicd.daemon')
     def GetDHCPClient(self):
         return self.dhcp_client
-    
+
     @dbus.service.method('org.wicd.daemon')
     def SetDHCPClient(self, client):
         print "Setting dhcp client to %i" % (int(client))
@@ -609,7 +622,7 @@ class ConnectionWizard(dbus.service.Object):
     @dbus.service.method('org.wicd.daemon')
     def GetLinkDetectionTool(self):
         return self.link_detect_tool
-    
+
 
     @dbus.service.method('org.wicd.daemon')
     def SetLinkDetectionTool(self, link_tool):
@@ -633,13 +646,13 @@ class ConnectionWizard(dbus.service.Object):
         config.read(self.app_conf)
         config.set("Settings", "flush_tool", flush_tool)
         config.write(open(self.app_conf, "w"))
-    
+
     @dbus.service.signal(dbus_interface='org.wicd.daemon', signature='')
     def LaunchChooser(self):
         """ Emits the wired profile chooser dbus signal. """
         print 'calling wired profile chooser'
         self.SetNeedWiredProfileChooser(True)
-        
+
     @dbus.service.method('org.wicd.daemon', in_signature='uav')
     def EmitStatusChanged(self, state, info):
         self.StatusChanged(state, info)
@@ -647,19 +660,19 @@ class ConnectionWizard(dbus.service.Object):
     @dbus.service.signal(dbus_interface='org.wicd.daemon', signature='uav')
     def StatusChanged(self, state, info):
         """ Emits a "status changed" dbus signal.
-        
+
         This D-Bus signal is emitted when the connection status changes.
-        
+
         """
         pass
-    
+
     @dbus.service.method('org.wicd.daemon')
     @dbus.service.signal(dbus_interface='org.wicd.daemon', signature='')
     def SendStartScanSignal(self):
         """ Emits a signal announcing a scan has started. """
         pass
 
-    
+
     @dbus.service.method('org.wicd.daemon')
     @dbus.service.signal(dbus_interface='org.wicd.daemon', signature='')
     def SendEndScanSignal(self):
@@ -677,10 +690,10 @@ class ConnectionWizard(dbus.service.Object):
     @dbus.service.method('org.wicd.daemon.wireless')
     def Scan(self):
         """ Scan for wireless networks.
-        
+
         Scans for wireless networks,optionally using a (hidden) essid
         set with SetHiddenNetworkESSID.
-        
+
         """
         if self.debug_mode:
             print 'scanning start'
@@ -698,13 +711,13 @@ class ConnectionWizard(dbus.service.Object):
     @dbus.service.method('org.wicd.daemon.wireless')
     def GetIwconfig(self):
         """ Calls and returns the output of iwconfig"""
-        return self.wifi.GetIwconfig()
+        return misc.to_unicode(self.wifi.GetIwconfig())
 
     @dbus.service.method('org.wicd.daemon.wireless')
     def GetNumberOfNetworks(self):
         """ Returns number of networks. """
         return len(self.LastScan)
-    
+
     @dbus.service.method('org.wicd.daemon.wireless')
     def GetApBssid(self):
         return self.wifi.GetBSSID()
@@ -723,9 +736,15 @@ class ConnectionWizard(dbus.service.Object):
         return status
 
     @dbus.service.method('org.wicd.daemon.wireless')
-    def GetWirelessProperty(self, networkid, property):
+    def GetWirelessProperty(self, networkid, prop):
         """ Retrieves wireless property from the network specified """
-        value = self.LastScan[networkid].get(property)
+        try:
+            value = self.LastScan[networkid].get(prop)
+        except IndexError:
+            if self.debug_mode:
+                print "GetWirelessProperty: Index error occured trying to " + \
+                      "retrieve property %s" % prop
+            value = ""
         try:
             value = misc.to_unicode(value)
         except:
@@ -733,14 +752,14 @@ class ConnectionWizard(dbus.service.Object):
         return value
 
     @dbus.service.method('org.wicd.daemon.wireless')
-    def SetWirelessProperty(self, networkid, property, value):
+    def SetWirelessProperty(self, networkid, prop, value):
         """ Sets property to value in network specified. """
         # We don't write script settings here.
-        if (property.strip()).endswith("script"):
+        if (prop.strip()).endswith("script"):
             print "Setting script properties through the daemon is not" \
-                   + " permitted."
+                  + " permitted."
             return False
-        self.LastScan[networkid][property] = misc.Noneify(value)
+        self.LastScan[networkid][prop] = misc.Noneify(value)
     #end function SetProperty
 
     @dbus.service.method('org.wicd.daemon.wireless')
@@ -748,17 +767,17 @@ class ConnectionWizard(dbus.service.Object):
         """ Returns an automatically detected wireless interface. """
         iface = self.wifi.DetectWirelessInterface()
         if iface:
-            print 'automatically detected wireless interface ' + iface
+            print 'Automatically detected wireless interface ' + iface
         else:
             print "Couldn't detect a wireless interface."
         return str(iface)
-    
+
     @dbus.service.method('org.wicd.daemon.wireless')
     def DisconnectWireless(self):
         """ Disconnects the wireless network. """
         self.SetForcedDisconnect(True)
         self.wifi.Disconnect()
-    
+
     @dbus.service.method('org.wicd.daemon.wireless')
     def IsWirelessUp(self):
         """ Returns a boolean specifying if wireless is up or down. """
@@ -767,12 +786,12 @@ class ConnectionWizard(dbus.service.Object):
     @dbus.service.method('org.wicd.daemon.wireless')
     def GetPrintableSignalStrength(self, iwconfig=None):
         """ Assigns a signal strength appropriate for display
-        
+
         This is used separately from the raw signal strength retrieving
         functions as a way to simply the strength polling process for
         the GUI and tray icon, by returning the strength that the user
         has requested to be displayed in wicd preferences.
-        
+
         """
         if self.GetSignalDisplayType() == 0:
             return self.GetCurrentSignalStrength(iwconfig)
@@ -813,7 +832,7 @@ class ConnectionWizard(dbus.service.Object):
         if self.debug_mode:
             print 'GetCurrentNetworkID: Returning -1, current network not found'
         return -1
-    
+
     @dbus.service.method('org.wicd.daemon.wireless')
     def ConnectWireless(self, id):
         """ Connects the the wireless network specified by i"""
@@ -824,7 +843,7 @@ class ConnectionWizard(dbus.service.Object):
         self.wifi.before_script = self.GetWirelessProperty(id, 'beforescript')
         self.wifi.after_script = self.GetWirelessProperty(id, 'afterscript')
         self.wifi.disconnect_script = self.GetWirelessProperty(id,
-                                                            'disconnectscript')
+                                                               'disconnectscript')
         print 'Connecting to wireless network ' + self.LastScan[id]['essid']
         return self.wifi.Connect(self.LastScan[id], debug=self.debug_mode)
 
@@ -896,24 +915,24 @@ class ConnectionWizard(dbus.service.Object):
             return False
 
     @dbus.service.method('org.wicd.daemon.wired')
-    def SetWiredProperty(self, property, value):
+    def SetWiredProperty(self, prop, value):
         """ Sets the given property to the given value. """
         if self.WiredNetwork:
-            if (property.strip()).endswith("script"):
+            if (prop.strip()).endswith("script"):
                 print "Setting script properties through the daemon" \
                       + " is not permitted."
                 return False
-            self.WiredNetwork[property] = misc.Noneify(value)
+            self.WiredNetwork[prop] = misc.Noneify(value)
             return True
         else:
             print 'SetWiredProperty: WiredNetwork does not exist'
             return False
 
     @dbus.service.method('org.wicd.daemon.wired')
-    def GetWiredProperty(self, property):
+    def GetWiredProperty(self, prop):
         """ Returns the requested wired property. """
         if self.WiredNetwork:
-            value = self.WiredNetwork.get(property)
+            value = self.WiredNetwork.get(prop)
             return value
         else:
             print 'GetWiredProperty: WiredNetwork does not exist'
@@ -926,7 +945,7 @@ class ConnectionWizard(dbus.service.Object):
             return True
         else:
             return False
-        
+
     @dbus.service.method('org.wicd.daemon.wired')
     def DisconnectWired(self):
         """ Disconnects the wired network. """
@@ -955,7 +974,7 @@ class ConnectionWizard(dbus.service.Object):
             return self.wired.CheckPluggedIn()
         else:
             return None
-        
+
     @dbus.service.method('org.wicd.daemon.wired')
     def DetectWiredInterface(self):
         """ Returns an automatically detected wired interface. """
@@ -1030,6 +1049,7 @@ class ConnectionWizard(dbus.service.Object):
             if config.has_option(profile, "lastused"):
                 if misc.to_bool(config.get(profile, "lastused")):
                     config.set(profile, "lastused", False)
+                    config.write(open(self.wired_conf, "w"))
                     self.SaveWiredNetworkProfile(profile)
 
     @dbus.service.method('org.wicd.daemon.config')
@@ -1042,6 +1062,7 @@ class ConnectionWizard(dbus.service.Object):
             if config.has_option(profile, "default"):
                 if misc.to_bool(config.get(profile, "default")):
                     config.set(profile, "default", False)
+                    config.write(open(self.wired_conf, "w"))
                     self.SaveWiredNetworkProfile(profile)
 
     @dbus.service.method('org.wicd.daemon.config')
@@ -1098,7 +1119,7 @@ class ConnectionWizard(dbus.service.Object):
         config.add_section(profilename)
         for x in self.WiredNetwork:
             config.set(profilename, x, self.WiredNetwork[x])
-        
+
         write_script_ent(profilename, config, "beforescript")
         write_script_ent(profilename, config, "afterscript")
         write_script_ent(profilename, config, "disconnectscript")
@@ -1139,7 +1160,7 @@ class ConnectionWizard(dbus.service.Object):
         def write_script_ent(prof, conf, script):
             if not conf.has_option(prof, script):
                 conf.set(prof, script, None)
-                
+
         config = ConfigParser.ConfigParser()
         config.read(self.wireless_conf)
         cur_network = self.LastScan[id]
@@ -1149,7 +1170,7 @@ class ConnectionWizard(dbus.service.Object):
         if config.has_section(bssid_key):
             config.remove_section(bssid_key)
         config.add_section(bssid_key)
-        
+
         # We want to write the essid in addition to bssid
         # sections if global settings are enabled.
         if cur_network["use_settings_globally"]:
@@ -1161,16 +1182,16 @@ class ConnectionWizard(dbus.service.Object):
             config.set(bssid_key, x, cur_network[x])
             if cur_network["use_settings_globally"]:
                 config.set(essid_key, x, cur_network[x])
-        
+
         write_script_ent(bssid_key, config, "beforescript")
         write_script_ent(bssid_key, config, "afterscript")
         write_script_ent(bssid_key, config, "disconnect")
-        
+
         if cur_network["use_settings_globally"]:
             write_script_ent(essid_key, config, "beforescript")
             write_script_ent(essid_key, config, "afterscript")
             write_script_ent(essid_key, config, "disconnect")
-            
+
         config.write(open(self.wireless_conf, "w"))
 
     @dbus.service.method('org.wicd.daemon.config')
@@ -1184,7 +1205,7 @@ class ConnectionWizard(dbus.service.Object):
         essid_key = "essid:" + cur_network["essid"]
         config = ConfigParser.ConfigParser()
         config.read(self.wireless_conf)
-        
+
         if not config.has_section(cur_network["bssid"]):
             config.add_section(cur_network["bssid"])
         config.set(cur_network["bssid"], option,
@@ -1196,7 +1217,7 @@ class ConnectionWizard(dbus.service.Object):
                 config.set(essid_key, option, str(cur_network[option]))
 
         config.write(open(self.wireless_conf, "w"))
-    
+
     @dbus.service.method('org.wicd.daemon.config')
     def RemoveGlobalEssidEntry(self, networkid):
         """ Removes the global entry for the networkid provided. """
@@ -1206,6 +1227,7 @@ class ConnectionWizard(dbus.service.Object):
         essid_key = "essid:" + cur_network["essid"]
         if config.has_section(essid_key):
             config.remove_section(essid_key)
+        config.write(open(self.wireless_conf, "w"))
 
     @dbus.service.method('org.wicd.daemon.config')
     def ReadWirelessNetworkProfile(self, id):
@@ -1217,8 +1239,8 @@ class ConnectionWizard(dbus.service.Object):
         bssid_key = cur_network["bssid"]
         if self.debug_mode:
             print bssid_key
-        if config.has_section(essid_key):
-            if config.get(essid_key, 'use_settings_globally'):
+        if config.has_section(essid_key)and \
+           misc.stringToNone(config.get(essid_key, 'use_settings_globally')):
                 return self._read_wireless_profile(config, cur_network, 
                                                    essid_key)
         elif config.has_section(bssid_key):
@@ -1226,15 +1248,18 @@ class ConnectionWizard(dbus.service.Object):
         else:
             cur_network["has_profile"] = False
             return "500: Profile Not Found"
-        
+
     def _read_wireless_profile(self, config, cur_network, section):
         cur_network["has_profile"] = True
 
         # Read the essid because we be needing to name those hidden
         # wireless networks now - but only read it if it is hidden.
         if cur_network["hidden"]:
-            cur_network["essid"] = misc.Noneify(config.get(section,
-                                                           "essid"))
+            if config.has_option(section, "essid"):
+                cur_network["essid"] = config.get(section, "essid")
+                if cur_network["essid"] in ["", "Hidden", "<hidden>"]:
+                    cur_network["essid"] = "<hidden>"
+
         for x in config.options(section):
             if not cur_network.has_key(x) or x.endswith("script"):
                 cur_network[x] = misc.Noneify(config.get(section, 
@@ -1242,6 +1267,9 @@ class ConnectionWizard(dbus.service.Object):
         for option in ['use_static_dns', 'use_global_dns', 'encryption',
                        'use_settings_globally']:
             cur_network[option] = bool(cur_network.get(option))
+        for key in cur_network:
+            cur_network[key] = misc.to_unicode(cur_network[key])
+            
         return "100: Loaded Profile"
 
     @dbus.service.method('org.wicd.daemon.config')
@@ -1259,14 +1287,14 @@ class ConnectionWizard(dbus.service.Object):
             config.set("Settings", width_str, width)
             config.set("Settings", height_str, height)
             config.write(open(self.app_conf, "w"))
-            
+
     @dbus.service.method('org.wicd.daemon.config')
     def ReadWindowSize(self, win_name):
         """Returns a list containing the desired default window size
-        
+
         Attempts to read the default size from the config file,
         and if that fails, returns a default of 605 x 400.
-        
+
         """
         default_width = default_height = width_str = height_str = -1
 
@@ -1317,9 +1345,9 @@ class ConnectionWizard(dbus.service.Object):
 
     def get_option(self, section, option, default=None):
         """ Method for returning an option from manager-settings.conf. 
-        
+
         This method will return a given option from a given section
-        
+
         """
         config = ConfigParser.ConfigParser()
         config.read(self.app_conf)
@@ -1337,17 +1365,17 @@ class ConnectionWizard(dbus.service.Object):
 
     def ReadConfig(self):
         """ Reads the manager-settings.conf file.
-        
+
         Reads the manager-settings.conf file and loads the stored
         values into memory.
-        
+
         """
         if os.path.isfile(self.app_conf):
             iface = self.DetectWirelessInterface()
             if not iface:
                 if self.debug_mode:
                     print "Failed to detect wireless interface, defaulting " + \
-                    "to wlan0, unless a config entry already exists."
+                          "to wlan0, unless a config entry already exists."
                 iface = "wlan0"
             self.SetWirelessInterface(self.get_option("Settings",
                                                       "wireless_interface",
@@ -1356,16 +1384,16 @@ class ConnectionWizard(dbus.service.Object):
             if not iface:
                 if self.debug_mode:
                     print "Failed to detect wired interface, defaulting " + \
-                           "to eth0, unless a config entry already exists."
+                          "to eth0, unless a config entry already exists."
                 iface = "eth0"
             self.SetWiredInterface(self.get_option("Settings",
                                                    "wired_interface",
-                                                   default="iface"))
+                                                   default=iface))
             self.SetWPADriver(self.get_option("Settings", "wpa_driver",
                                               default="wext"))
             self.SetAlwaysShowWiredInterface(self.get_option("Settings",
-                                                  "always_show_wired_interface",
-                                                  default=False))
+                                                             "always_show_wired_interface",
+                                                             default=False))
 
             self.SetUseGlobalDNS(self.get_option("Settings", "use_global_dns",
                                                  default=False))
@@ -1374,7 +1402,7 @@ class ConnectionWizard(dbus.service.Object):
             dns3 = self.get_option("Settings", "global_dns_3", default='None')
             self.SetGlobalDNS(dns1, dns2, dns3)
             self.SetAutoReconnect(self.get_option("Settings", "auto_reconnect",
-                                                  default=False))
+                                                  default=True))
             self.SetDebugMode(self.get_option("Settings", "debug_mode",
                                               default=False))
 
@@ -1400,7 +1428,7 @@ class ConnectionWizard(dbus.service.Object):
             config.set("Settings", "wireless_interface", "wlan0")
             config.set("Settings", "wired_interface", "eth0")
             config.set("Settings", "always_show_wired_interface", "False")
-            config.set("Settings", "auto_reconnect", "False")
+            config.set("Settings", "auto_reconnect", "True")
             config.set("Settings", "debug_mode", "False")
             config.set("Settings", "wired_connect_mode", "1")
             config.set("Settings", "signal_display_type", "0")
@@ -1416,6 +1444,12 @@ class ConnectionWizard(dbus.service.Object):
             else:
                 print "Couldn't detect a wireless interface, using wlan0..."
                 config.set("Settings", "wireless_interface", "wlan0")
+            iface = self.DetectWiredInterface()
+            if iface is not None:
+                config.set("Settings", "wired_interface", iface)
+            else:
+                print "Couldn't detect a wired interface, using eth0..."
+                config.set("Settings", "wired_interface", "eth0")
             config.set("Settings", "wpa_driver", "wext")
             config.write(open(self.app_conf, "w"))
             self.SetWirelessInterface(config.get("Settings",
@@ -1429,7 +1463,7 @@ class ConnectionWizard(dbus.service.Object):
                                                  "link_detect_tool"))
             self.SetFlushTool(config.get("Settings", "flush_tool"))
             self.SetAlwaysShowWiredInterface(False)
-            self.SetAutoReconnect(False)
+            self.SetAutoReconnect(True)
             self.SetDebugMode(False)
             self.SetWiredAutoConnectMethod(1)
             self.SetSignalDisplayType(0)
@@ -1469,11 +1503,12 @@ class ConnectionWizard(dbus.service.Object):
 
         print "Using wired interface..." + self.GetWiredInterface()
         print "Using wireless interface..." + self.GetWirelessInterface()
-                
+
 
 def usage():
+    # VERSIONNUMBER
     print """
-wicd 1.5.2
+wicd 1.5.6
 wireless (and wired) connection daemon.
 
 Arguments:
@@ -1526,13 +1561,13 @@ def daemonize():
     except OSError, e:
         print >> sys.stderr, "Fork #2 failed: %d (%s)" % (e.errno, e.strerror)
         sys.exit(1)
-        
+
     sys.stdout.flush()
     sys.stderr.flush()
     os.close(sys.__stdin__.fileno())
     os.close(sys.__stdout__.fileno())
     os.close(sys.__stderr__.fileno())
-    
+
     # stdin always from /dev/null
     sys.stdin = open('/dev/null', 'r')
 
@@ -1552,8 +1587,8 @@ def main(argv):
 
     try:
         opts, args = getopt.getopt(sys.argv[1:], 'fenoah',
-                ['help', 'no-daemon', 'no-poll', 'no-stderr', 'no-stdout',
-               'no-autoconnect'])
+                                   ['help', 'no-daemon', 'no-poll', 'no-stderr', 'no-stdout',
+                                    'no-autoconnect'])
     except getopt.GetoptError:
         # Print help information and exit
         usage()
@@ -1586,16 +1621,16 @@ def main(argv):
     print '---------------------------'
 
     # Open the DBUS session
-    
+
     d_bus_name = dbus.service.BusName('org.wicd.daemon', bus=dbus.SystemBus())
     obj = ConnectionWizard(d_bus_name, auto_connect=auto_connect)
 
     gobject.threads_init()
     if not no_poll:
         (child_pid, x, x, x) = gobject.spawn_async([wpath.lib + "monitor.py"], 
-                                       flags=gobject.SPAWN_CHILD_INHERITS_STDIN)
+                                                   flags=gobject.SPAWN_CHILD_INHERITS_STDIN)
         signal.signal(signal.SIGTERM, sigterm_caught)
-    
+
     # Enter the main loop
     mainloop = gobject.MainLoop()
     mainloop.run()
