@@ -34,13 +34,14 @@ class WirelessInterface() -- Control a wireless network interface.
 import re
 import os
 import time
+from string import maketrans, translate, punctuation
 
 import wicd.wpath as wpath
 import wicd.misc as misc
 
 # Compile the regex patterns that will be used to search the output of iwlist
 # scan for info these are well tested, should work on most cards
-essid_pattern       = re.compile('.*ESSID:"(.*?)"\n', re.I | re.M  | re.S)
+essid_pattern       = re.compile('.*ESSID:"?(.*?)"?\s*\n', re.I | re.M  | re.S)
 ap_mac_pattern      = re.compile('.*Address: (.*?)\n', re.I | re.M  | re.S)
 channel_pattern     = re.compile('.*Channel:? ?(\d\d?)', re.I | re.M  | re.S)
 strength_pattern    = re.compile('.*Quality:?=? ?(\d+)\s*/?\s*(\d*)', re.I | re.M  | re.S)
@@ -64,19 +65,21 @@ auth_pattern        = re.compile('.*wpa_state=(.*?)\n', re.I | re.M  | re.S)
 
 RALINK_DRIVER = 'ralink legacy'
 
+blacklist_strict = punctuation.replace("-", "") + " "
+blacklist_norm = ";`$!*|><&\\"
+blank_trans = maketrans("", "")
+
 def _sanitize_string(string):
-    blacklist = [';', '`', '$', '!', '*', '|', '>', '<']
-    new_string = []
-    
-    if not string:
+    if string:
+        return translate(str(string), blank_trans, blacklist_norm)
+    else:
+        return string
+
+def _sanitize_string_strict(string):
+    if string:
+        return translate(str(string), blank_trans, blacklist_strict)
+    else:
         return string
-        
-    for c in string:
-        if c in blacklist:
-            new_string.append("\\" + c)
-        else:
-            new_string.append(c)
-    return ''.join(new_string)
 
 def SetDNS(dns1=None, dns2=None, dns3=None):
     """ Set the DNS of the system to the specified DNS servers.
@@ -85,8 +88,8 @@ def SetDNS(dns1=None, dns2=None, dns3=None):
 
     Keyword arguments:
     dns1 -- IP address of DNS server 1
-    dns2 -- IP address of DNS server 1
-    dns3 -- IP address of DNS server 1
+    dns2 -- IP address of DNS server 2
+    dns3 -- IP address of DNS server 3
 
     """
     resolv = open("/etc/resolv.conf","w")
@@ -112,7 +115,7 @@ def GetDefaultGateway():
         if words[0] == '0.0.0.0':
             gateway = words[1]
             break
-        
+
     if not gateway:
         print 'couldn\'t retrieve default gateway from route -n'
     return gateway
@@ -142,21 +145,21 @@ def _fast_get_wifi_interfaces():
     """ Tries to get a wireless interface using /sys/class/net. """
     dev_dir = '/sys/class/net/'
     ifnames = []
-    
+
     ifnames = [iface for iface in os.listdir(dev_dir) if os.path.isdir(dev_dir + iface) \
                and 'wireless' in os.listdir(dev_dir + iface)]
-    
+
     return bool(ifnames) and ifnames[0] or None
 
 def GetWiredInterfaces():
     basedir = '/sys/class/net/'
-    return [iface for iface in os.listdir(basedir) if not 'wireless' 
-            in os.listdir(basedir + iface) and 
+    return [iface for iface in os.listdir(basedir) if os.path.isdir(basedir + iface) \
+            and not 'wireless' in os.listdir(basedir + iface) and
             open(basedir + iface + "/type").readlines()[0].strip() == "1"]
 
 class Interface(object):
     """ Control a network interface. """
-    def __init__(self, iface, verbose=False):
+    def __init__(self, iface, dhcp_client, flush_tool, verbose=False):
         """ Initialise the object.
 
         Keyword arguments:
@@ -164,42 +167,42 @@ class Interface(object):
         verbose -- whether to print every command run
 
         """
-        self.iface = _sanitize_string(iface)
+        self.iface = _sanitize_string_strict(iface)
         self.verbose = verbose
-        self.DHCP_CLIENT = None
+        self.DHCP_CLIENT = dhcp_client
         self.DHCP_CMD = None
         self.DHCP_RELEASE = None
         self.MIITOOL_FOUND = False
         self.ETHTOOL_FOUND = False
         self.IP_FOUND = False
-        self.flush_tool = None
+        self.flush_tool = flush_tool
         self.link_detect = None
         self.Check()
-        
+
     def SetDebugMode(self, value):
         """ If True, verbose output is enabled. """
         self.verbose = value
 
     def SetInterface(self, iface):
         """ Sets the interface.
-        
+
         Keyword arguments:
         iface -- the name of the interface.
-        
+
         """
-        self.iface = _sanitize_string(str(iface))
-        
+        self.iface = _sanitize_string_strict(str(iface))
+
     def _find_client_path(self, client):
         """ Determines the full path for the given program.
-        
+
         Searches a hardcoded list of paths for a given program name.
-        
+
         Keyword arguments:
         client -- The name of the program to search for
-        
+
         Returns:
         The full path of the program or None
-        
+
         """
         paths = ['/sbin/', '/usr/sbin/', '/bin/', '/usr/bin/',
                  '/usr/local/sbin/', '/usr/local/bin/']
@@ -207,32 +210,17 @@ class Interface(object):
             if os.access("%s%s" % (path, client), os.F_OK):
                 return "%s%s" % (path, client)
         if self.verbose:
-            "WARNING: No path found for %s"  % (client)
+            print "WARNING: No path found for %s"  % (client)
         return None
-        
-    def _client_found(self, client):
-        """ Searches for the existence of the given program in PATH.
-        
-        Uses "which" to determine if a given program exists in PATH.
-        
-        Returns:
-        True if the program exists, False otherwise.
-        
-        """
-        client = _sanitize_string(client)
-        output = misc.Run("which " + client)
-        if output and not ("no " + client) in output:
-            return True
-        return False
 
     def CheckDHCP(self):
-        """ Check for a valid DHCP client. 
-        
+        """ Check for a valid DHCP client.
+
         Checks for the existence of a supported DHCP client.  If one is
         found, the appropriate values for DHCP_CMD, DHCP_RELEASE, and
         DHCP_CLIENT are set.  If a supported client is not found, a
         warning is printed.
-        
+
         """
         def get_client_name(cl):
             """ Converts the integer value for a dhcp client to a string. """
@@ -243,9 +231,7 @@ class Interface(object):
             else:
                 client = "pump"
             return client
-        
-        print 'checking dhcp...'
-        
+
         if self.DHCP_CLIENT:
             dhcp_client = get_client_name(self.DHCP_CLIENT)
             dhcp_path = self._find_client_path(dhcp_client)
@@ -261,7 +247,7 @@ class Interface(object):
                 if dhcp_path:
                     dhcp_client = client
                     break
-    
+
         if not dhcp_client:
             print "WARNING: No supported DHCP Client could be found!"
             return
@@ -285,7 +271,7 @@ class Interface(object):
         self.DHCP_CMD = dhcp_cmd
         self.DHCP_RELEASE = dhcp_release
         self.DHCP_CLIENT = dhcp_client
-    
+
     def CheckWiredTools(self):
         """ Check for the existence of ethtool and mii-tool. """
         miitool_path = self._find_client_path("mii-tool")
@@ -295,7 +281,7 @@ class Interface(object):
         else:
             self.miitool_cmd = None
             self.MIITOOL_FOUND = False
-        
+
         ethtool_path = self._find_client_path("ethtool")
         if ethtool_path:
             self.ethtool_cmd = ethtool_path
@@ -303,7 +289,7 @@ class Interface(object):
         else:
             self.ethtool_cmd = None
             self.ETHTOOL_FOUND = False
-            
+
     def CheckWirelessTools(self):
         """ Check for the existence of wpa_cli """
         wpa_cli_path = self._find_client_path("wpa_cli")
@@ -319,7 +305,7 @@ class Interface(object):
         self.CheckDHCP()
         self.CheckWiredTools()
         self.CheckWirelessTools()
-        
+
         ip_path = self._find_client_path("ip")
         if ip_path:
             self.ip_cmd = ip_path
@@ -330,10 +316,10 @@ class Interface(object):
 
     def Up(self):
         """ Bring the network interface up.
-        
+
         Returns:
         True
-        
+
         """
         if not self.iface: return False
         cmd = 'ifconfig ' + self.iface + ' up'
@@ -342,11 +328,11 @@ class Interface(object):
         return True
 
     def Down(self):
-        """ Take down the network interface. 
-        
+        """ Take down the network interface.
+
         Returns:
         True
-        
+
         """
         if not self.iface: return False
         cmd = 'ifconfig ' + self.iface + ' down'
@@ -365,14 +351,14 @@ class Interface(object):
         """
         if not self.iface:
             return
-        
+
         for val in [ip, netmask, broadcast]:
             if not val:
                 continue
             if not misc.IsValidIP(val):
                 print 'WARNING: Invalid IP address found, aborting!'
                 return False
-        
+
         cmd = ''.join(['ifconfig ', self.iface, ' '])
         if ip:
             cmd = ''.join([cmd, ip, ' '])
@@ -385,20 +371,20 @@ class Interface(object):
 
     def _parse_dhclient(self, pipe):
         """ Parse the output of dhclient.
-        
+
         Parses the output of dhclient and returns the status of
         the connection attempt.
 
         Keyword arguments:
         pipe -- stdout pipe to the dhcpcd process.
-        
+
         Returns:
         'success' if succesful', an error code string otherwise.
-        
+
         """
         dhclient_complete = False
         dhclient_success = False
-        
+
         while not dhclient_complete:
             line = pipe.readline()
             if line == '':  # Empty string means dhclient is done.
@@ -408,22 +394,22 @@ class Interface(object):
             if line.startswith('bound'):
                 dhclient_success = True
                 dhclient_complete = True
-                
+
         return self._check_dhcp_result(dhclient_success)
-        
+
     def _parse_pump(self, pipe):
         """ Determines if obtaining an IP using pump succeeded.
 
         Keyword arguments:
         pipe -- stdout pipe to the dhcpcd process.
-        
+
         Returns:
         'success' if succesful, an error code string otherwise.
-        
+
         """
         pump_complete = False
         pump_success = True
-        
+
         while not pump_complete:
             line = pipe.readline()
             if line == '':
@@ -432,22 +418,22 @@ class Interface(object):
                 pump_success = False
                 pump_complete = True
             print line
-            
+
         return self._check_dhcp_result(pump_success)
 
     def _parse_dhcpcd(self, pipe):
         """ Determines if obtaining an IP using dhcpcd succeeded.
-        
+
         Keyword arguments:
         pipe -- stdout pipe to the dhcpcd process.
-        
+
         Returns:
         'success' if succesful', an error code string otherwise.
-        
+
         """
         dhcpcd_complete = False
         dhcpcd_success = True
-        
+
         while not dhcpcd_complete:
             line = pipe.readline()
             if line.startswith("Error"):
@@ -456,18 +442,18 @@ class Interface(object):
             elif line == '':
                 dhcpcd_complete = True
             print line
-            
+
         return self._check_dhcp_result(dhcpcd_success)
-        
+
     def _check_dhcp_result(self, success):
-        """ Print and return the correct DHCP connection result. 
-        
+        """ Print and return the correct DHCP connection result.
+
         Keyword Arguents:
         success -- boolean specifying if DHCP was succesful.
-        
+
         Returns:
         'success' if success == True, 'dhcp_failed' otherwise.
-        
+
         """
         if success:
             print 'DHCP connection successful'
@@ -475,29 +461,29 @@ class Interface(object):
         else:
             print 'DHCP connection failed'
             return 'dhcp_failed'
-            
+
     def StartDHCP(self):
         """ Start the DHCP client to obtain an IP address.
-        
+
         Returns:
         A string representing the result of the DHCP command.  See
         _check_dhcp_result for the possible values.
-        
+
         """
         if not self.iface: return False
-        
+
         cmd = self.DHCP_CMD + " " + self.iface
         if self.verbose: print cmd
         pipe = misc.Run(cmd, include_stderr=True, return_pipe=True)
-        
-        DHCP_CLIENT = self.DHCP_CLIENT        
+
+        DHCP_CLIENT = self.DHCP_CLIENT
         if DHCP_CLIENT == misc.DHCLIENT:
             return self._parse_dhclient(pipe)
         elif DHCP_CLIENT == misc.PUMP:
             return self._parse_pump(pipe)
         elif DHCP_CLIENT == misc.DHCPCD:
             return self._parse_dhcpcd(pipe)
-    
+
     def ReleaseDHCP(self):
         """ Release the DHCP lease for this interface. """
         if not self.iface: return False
@@ -541,22 +527,35 @@ class Interface(object):
         if self.verbose: print cmd
         output = misc.Run(cmd)
         return misc.RunRegex(ip_pattern, output)
-    
-    def IsUp(self):
+
+    def IsUp(self, ifconfig=None):
         """ Determines if the interface is up.
-        
+
         Returns:
         True if the interface is up, False otherwise.
-        
+
         """
         if not self.iface: return False
-        cmd = "ifconfig " + self.iface
-        output = misc.Run(cmd)
+        flags_file = '/sys/class/net/%s/flags' % self.iface
+        try:
+            flags = open(flags_file, "r").read().strip()
+        except IOError:
+            print "Could not open %s, using ifconfig to determine status" % flags_file
+            return self._slow_is_up(ifconfig)
+        return bool(int(flags, 16) & 0x1)
+    
+    def _slow_is_up(self, ifconfig):
+        """ Determine if an interface is up using ifconfig. """
+        if not ifconfig:
+            cmd = "ifconfig " + self.iface
+            output = misc.Run(cmd)
+        else:
+            output = ifconfig
         lines = output.split('\n')
         if len(lines) < 5:
             return False
-        
-        for line in lines[1:4]:
+
+        for line in lines[1:]:
             if line.strip().startswith('UP'):
                 return True
             
@@ -565,7 +564,8 @@ class Interface(object):
 
 class WiredInterface(Interface):
     """ Control a wired network interface. """
-    def __init__(self, iface, verbose=False):
+    def __init__(self, iface, dhcp_client, flush_tool, link_detect, 
+                 verbose=False):
         """ Initialise the wired network interface class.
 
         Keyword arguments:
@@ -573,36 +573,62 @@ class WiredInterface(Interface):
         verbose -- print all commands
 
         """
-        Interface.__init__(self, iface, verbose)
+        Interface.__init__(self, iface, dhcp_client, flush_tool, verbose)
+        self.link_detect = None
 
     def GetPluggedIn(self):
         """ Get the current physical connection state.
-        
+
         The method will first attempt to use ethtool do determine
         physical connection state.  Should ethtool fail to run properly,
         mii-tool will be used instead.
-        
+
         Returns:
         True if a link is detected, False otherwise.
-        
+
         """
         if not self.iface:
             return False
+        # check for link using /sys/class/net/iface/carrier
+        # is usually more accurate
+        sys_device = '/sys/class/net/%s/' % self.iface
+        carrier_path = sys_device + 'carrier'
+        if not self.IsUp():
+            MAX_TRIES = 3
+            tries = 0
+            self.Up()
+            while True:
+                tries += 1
+                time.sleep(2)
+                if self.IsUp() or tries > MAX_TRIES: break
+      
+        if os.path.exists(carrier_path):
+            carrier = open(carrier_path, 'r')
+            try:
+                link = carrier.read().strip()
+                link = int(link)
+                if link == 1:
+                    return True
+                elif link == 0:
+                    return False
+            except (IOError, ValueError, TypeError):
+                print 'Error checking link using /sys/class/net/%s/carrier' % self.iface
+                
         if self.ETHTOOL_FOUND and self.link_detect != misc.MIITOOL:
             return self._eth_get_plugged_in()
         elif self.MIITOOL_FOUND:
             return self._mii_get_plugged_in()
         else:
-            print 'Error: No way of checking for a wired connection. Make \
-                   sure that either mii-tool or ethtool is installed.'
+            print 'Error: No way of checking for a wired connection. Make ' + \
+                   'sure that either mii-tool or ethtool is installed.'
             return False
 
     def _eth_get_plugged_in(self):
         """ Use ethtool to determine the physical connection state.
-        
+
         Returns:
         True if a link is detected, False otherwise.
-        
+
         """
         link_tool = 'ethtool'
         if not self.IsUp():
@@ -610,38 +636,39 @@ class WiredInterface(Interface):
             self.Up()
             time.sleep(6)
         tool_data = misc.Run(link_tool + ' ' + self.iface, True)
-        if misc.RunRegex(re.compile('(Link detected: yes)', re.I | re.M  | 
+        if misc.RunRegex(re.compile('(Link detected: yes)', re.I | re.M  |
                                     re.S), tool_data) is not None:
             return True
         else:
             return False
-    
+
     def _mii_get_plugged_in(self):
-        """ Use mii-tool to determine the physical connection state. 
-                
+        """ Use mii-tool to determine the physical connection state.
+
         Returns:
         True if a link is detected, False otherwise.
-        
+
         """
         link_tool = 'mii-tool'
         tool_data = misc.Run(link_tool + ' ' + self.iface, True)
-        if misc.RunRegex(re.compile('(Invalid argument)', re.I | re.M  | re.S), 
+        if misc.RunRegex(re.compile('(Invalid argument)', re.I | re.M  | re.S),
                          tool_data) is not None:
             print 'Wired Interface is down, putting it up'
             self.Up()
             time.sleep(4)
             tool_data = misc.Run(link_tool + ' ' + self.iface, True)
-        
+
         if misc.RunRegex(re.compile('(link ok)', re.I | re.M | re.S),
                          tool_data) is not None:
             return True
         else:
             return False
-        
+
 
 class WirelessInterface(Interface):
     """ Control a wireless network interface. """
-    def __init__(self, iface, verbose=False, wpa_driver='wext'):
+    def __init__(self, iface, dhcp_client, flush_tool, verbose=False, 
+                 wpa_driver='wext'):
         """ Initialise the wireless network interface class.
 
         Keyword arguments:
@@ -649,9 +676,9 @@ class WirelessInterface(Interface):
         verbose -- print all commands
 
         """
-        Interface.__init__(self, iface, verbose)
+        Interface.__init__(self, iface, dhcp_client, flush_tool, verbose)
         self.wpa_driver = wpa_driver
-        
+
     def SetWpaDriver(self, driver):
         """ Sets the wpa_driver. """
         self.wpa_driver = _sanitize_string(driver)
@@ -663,9 +690,8 @@ class WirelessInterface(Interface):
         essid -- essid to set the interface to
 
         """
-        essid = _sanitize_string(essid)
-        cmd = 'iwconfig %s essid "%s"' % (self.iface, essid)
-        if self.verbose: print cmd
+        cmd = ['iwconfig', self.iface, 'essid', essid]
+        if self.verbose: print str(cmd)
         misc.Run(cmd)
 
     def StopWPA(self):
@@ -676,10 +702,10 @@ class WirelessInterface(Interface):
 
     def GetKillSwitchStatus(self):
         """ Determines if the wireless killswitch is enabled.
-        
+
         Returns:
         True if the killswitch is enabled, False otherwise.
-        
+
         """
         if not self.iface: return False
         output = misc.Run("iwconfig " + self.iface)
@@ -691,7 +717,7 @@ class WirelessInterface(Interface):
             radiostatus = False
 
         return radiostatus
-    
+
     def GetIwconfig(self):
         """ Returns the output of iwconfig for this interface. """
         if not self.iface: return ""
@@ -707,7 +733,6 @@ class WirelessInterface(Interface):
         cmd = 'iwlist ' + self.iface + ' scan'
         if self.verbose: print cmd
         results = misc.Run(cmd)
-
         # Split the networks apart, using Cell as our split point
         # this way we can look at only one network at a time.
         # The spaces around '   Cell ' are to minimize the chance that someone
@@ -756,21 +781,62 @@ class WirelessInterface(Interface):
             ret = freq_dict[freq]
         except KeyError:
             print "Couldn't determine channel number for frequency: " + str(freq)
-        
+
         return ret
 
     def _GetRalinkInfo(self):
         """ Get a network info list used for ralink drivers
-
+    
         Calls iwpriv <wireless interface> get_site_survey, which
         on some ralink cards will return encryption and signal
         strength info for wireless networks in the area.
-
+    
         """
         iwpriv = misc.Run('iwpriv ' + self.iface + ' get_site_survey')
-        lines = iwpriv.splitlines()
-        lines = lines[2:]
-        return lines
+        if self.verbose: print str(iwpriv)
+        lines = iwpriv.splitlines()[2:]
+        aps = {}
+        patt = re.compile("((?:[0-9A-Z]{2}:){5}[0-9A-Z]{2})")
+        for x in lines:
+            ap = {}
+            info = x.split("   ")
+            info = filter(None, [x.strip() for x in info])
+            if len(info) < 5:
+                continue
+            if re.match(patt, info[2].upper()):
+                bssid = info[2].upper()
+                offset = -1
+            elif re.match(patt, info[3].upper()):
+                bssid = info[3].upper()
+                offset = 0
+            else:  # Invalid
+                print 'Invalid iwpriv line.  Skipping it.'
+                continue
+            ap['nettype'] = info[-1]
+            ap['strength'] = info[1]
+            if info[4 + offset] == 'WEP':
+                ap['encryption_method'] = 'WEP'
+                ap['encryptype'] = 'WEP'
+                ap['keyname'] = 'Key1'
+                ap['authmode'] = info[5 + offset]
+            elif info[5 + offset] in ['WPA-PSK', 'WPA']:
+                ap['encryption_method'] = 'WPA'
+                ap['authmode'] = "WPAPSK"
+                ap['keyname'] = "WPAPSK"
+                ap['encryptype'] = info[4 + offset]
+            elif info[5 + offset] == 'WPA2-PSK':
+                ap['encryption_method'] = 'WPA2'
+                ap['authmode'] ="WPA2PSK"
+                ap['keyname'] = "WPA2PSK"
+                ap['encryptype'] = info[4 + offset]
+            elif info[4 + offset] == "NONE":
+                ap['encryption_method'] = None
+            else:
+                print "Unknown AuthMode, can't assign encryption_method!"
+                ap['encryption_method'] = 'Unknown'
+            aps[bssid] = ap
+        if self.verbose: print str(aps)
+        return aps
 
     def _ParseAccessPoint(self, cell, ralink_info):
         """ Parse a single cell from the output of iwlist.
@@ -794,7 +860,7 @@ class WirelessInterface(Interface):
             print 'Unicode problem with current network essid, ignoring!!'
             return None
         if ap['essid'] in ['<hidden>', ""]:
-            ap['essid'] = 'Hidden'
+            ap['essid'] = '<hidden>'
             ap['hidden'] = True
         else:
             ap['hidden'] = False
@@ -839,7 +905,11 @@ class WirelessInterface(Interface):
         if (strength_pattern.match(cell)):
             [(strength, max_strength)] = strength_pattern.findall(cell)
             if max_strength:
-                ap["quality"] = 100 * int(strength) // int(max_strength)
+                if int(max_strength) != 0:
+                    ap["quality"] = 100 * int(strength) // int(max_strength)
+                else:
+                    # prevent a divide by zero error
+                    ap['quality'] = int(strength)
             else:
                 ap["quality"] = int(strength)
         elif misc.RunRegex(altstrength_pattern,cell):
@@ -861,39 +931,21 @@ class WirelessInterface(Interface):
 
         Keyword arguments:
         ap -- array containing info about the current access point
-        ralink_info -- string containing available network info
+        ralink_info -- dict containing available network info
         cell -- string containing cell information
 
         Returns:
         Updated array containing info about the current access point
 
         """
-        lines = ralink_info
-        for x in lines:  # Iterate through all networks found
-            info = x.split()
-            # Make sure we read in a valid entry
-            if len(info) < 5 or info == None or info == '':
-                break
-            if info[2] == ap['essid']:
-                if misc.RunRegex(wep_pattern, cell) == 'on':
-                    ap['encryption'] = True
-                    if info[5] == 'WEP' or (
-                            (info[5] == 'OPEN' or info[5] == 'SHARED') and
-                        info[4] == 'WEP'):
-                        ap['encryption_method'] = 'WEP'
-                    elif info[5] == 'WPA-PSK':
-                        ap['encryption_method'] = 'WPA'
-                    elif info[5] == 'WPA2-PSK':
-                        ap['encryption_method'] = 'WPA2'
-                    else:
-                        print 'Unknown AuthMode, can\'t assign encryption_method!!'
-                        ap['encryption_method'] = 'Unknown'
-                else:
-                    ap['encryption'] = False
-
-                # Set signal strength here (in dBm, not %),
-                # ralink drivers don't return link quality
-                ap['strength'] = info[1]
+        if ralink_info.has_key(ap['bssid']):
+            info = ralink_info[ap['bssid']]
+            for key in info.keys():
+                ap[key] = info[key]
+            if misc.RunRegex(wep_pattern, cell) == 'on':
+                ap['encryption'] = True
+            else:
+                ap['encryption'] = False
         return ap
 
     def SetMode(self, mode):
@@ -904,7 +956,7 @@ class WirelessInterface(Interface):
 
         """
         if not self.iface: return False
-        mode = _sanitize_string(mode)
+        mode = _sanitize_string_strict(mode)
         if mode.lower() == 'master':
             mode = 'managed'
         cmd = 'iwconfig %s mode %s' % (self.iface, mode)
@@ -922,7 +974,7 @@ class WirelessInterface(Interface):
         if not channel.isdigit():
             print 'WARNING: Invalid channel found.  Aborting!'
             return False
-        
+
         cmd = 'iwconfig %s channel %s' % (self.iface, str(channel))
         if self.verbose: print cmd
         misc.Run(cmd)
@@ -935,7 +987,6 @@ class WirelessInterface(Interface):
 
         """
         if not self.iface: return False
-        key = _sanitize_string(key)
         cmd = 'iwconfig %s key %s' % (self.iface, key)
         if self.verbose: print cmd
         misc.Run(cmd)
@@ -950,13 +1001,27 @@ class WirelessInterface(Interface):
 
         """
         if not self.iface: return False
-        cmd = 'iwconfig %s essid "%s"' % (self.iface, essid)
+        cmd = ['iwconfig', self.iface, 'essid', essid]
         if channel:
-            cmd = ''.join([cmd, ' channel ', str(channel)])
+            cmd.extend(['channel', str(channel)])
         if bssid:
-            cmd = ''.join([cmd, ' ap ', bssid])
+            cmd.extend(['ap', bssid])
         if self.verbose: print cmd
         misc.Run(cmd)
+        
+    def GeneratePSK(self, network):
+        """ Generate a PSK using wpa_passphrase. 
+
+        Keyword arguments:
+        network -- dictionary containing network info
+        
+        """
+        wpa_pass_path = misc.find_path('wpa_passphrase')
+        if not wpa_pass_path: return None
+        key_pattern = re.compile('network={.*?\spsk=(.*?)\n}.*',
+                                 re.I | re.M  | re.S)
+        cmd = ' '.join([wpa_pass_path, network['essid'], network['key']])
+        return misc.RunRegex(key_pattern, misc.Run(cmd))
 
     def Authenticate(self, network):
         """ Authenticate with the specified wireless network.
@@ -969,9 +1034,9 @@ class WirelessInterface(Interface):
         if self.wpa_driver == RALINK_DRIVER:
             self._AuthenticateRalinkLegacy(network)
         else:
-            cmd = ''.join(['wpa_supplicant -B -i ', self.iface, ' -c "',
+            cmd = ''.join(['wpa_supplicant -B -i ', self.iface, ' -c ',
                        wpath.networks, network['bssid'].replace(':','').lower(),
-                       '" -D ', self.wpa_driver])
+                       ' -D ', self.wpa_driver])
             if self.verbose: print cmd
             misc.Run(cmd)
 
@@ -984,10 +1049,10 @@ class WirelessInterface(Interface):
             NOTE: It's possible this could return False,
             though in reality wpa_supplicant just isn't
             finished yet.
-            
+
             Keyword arguments:
             auth_time -- The time at which authentication began.
-            
+
             Returns:
             True if wpa_supplicant authenticated succesfully,
             False otherwise.
@@ -997,8 +1062,9 @@ class WirelessInterface(Interface):
         if self.wpa_driver == RALINK_DRIVER or not self.WPA_CLI_FOUND:
             return True
 
-        MAX_TIME = 15
+        MAX_TIME = 40
         MAX_DISCONNECTED_TIME = 3
+        disconnected_time = 0
         while (time.time() - auth_time) < MAX_TIME:
             cmd = 'wpa_cli -i ' + self.iface + ' status'
             output = misc.Run(cmd)
@@ -1010,27 +1076,31 @@ class WirelessInterface(Interface):
                 return False
             if result == "COMPLETED":
                 return True
-            elif result == "DISCONNECTED" and \
-                 (time.time() - auth_time) > MAX_DISCONNECTED_TIME:
-                # Force a rescan to get wpa_supplicant moving again.
-                self._ForceSupplicantScan()
-                MAX_TIME += 5
+            elif result == "DISCONNECTED":
+                disconnected_time += 1
+                if disconnected_time > MAX_DISCONNECTED_TIME:
+                    disconnected_time = 0
+                    # Force a rescan to get wpa_supplicant moving again.
+                    self._ForceSupplicantScan()
+                    MAX_TIME += 5
+            else:
+                disconnected_time = 0
             time.sleep(1)
 
         print 'wpa_supplicant authentication may have failed.'
         return False
-        
+
 
     def _ForceSupplicantScan(self):
         """ Force wpa_supplicant to rescan available networks.
-    
+
         This function forces wpa_supplicant to rescan.
         This works around authentication validation sometimes failing for
-        wpa_supplicant because it remains in a DISCONNECTED state for 
+        wpa_supplicant because it remains in a DISCONNECTED state for
         quite a while, after which a rescan is required, and then
         attempting to authenticate.  This whole process takes a long
         time, so we manually speed it up if we see it happening.
-        
+
         """
         print 'wpa_supplicant rescan forced...'
         cmd = 'wpa_cli -i' + self.iface + ' scan'
@@ -1047,51 +1117,30 @@ class WirelessInterface(Interface):
 
         """
         if network.get('key') != None:
-            lines = self._GetRalinkInfo()
-            for x in lines:
-                info = x.split()
-                if len(info) < 5:
-                    break
-                if info[2] == network.get('essid'):
-                    if info[5] == 'WEP' or (info[5] == 'OPEN' and \
-                                            info[4] == 'WEP'):
-                        print 'Setting up WEP'
-                        cmd = ''.join(['iwconfig ', self.iface, ' key ',
-                                      network.get('key')])
-                        if self.verbose: print cmd
-                        misc.Run(cmd)
-                    else:
-                        if info[5] == 'SHARED' and info[4] == 'WEP':
-                            print 'Setting up WEP'
-                            auth_mode = 'SHARED'
-                            key_name = 'Key1'
-                        elif info[5] == 'WPA-PSK':
-                            print 'Setting up WPA-PSK'
-                            auth_mode = 'WPAPSK'
-                            key_name = 'WPAPSK'
-                        elif info[5] == 'WPA2-PSK':
-                            print 'Setting up WPA2-PSK'
-                            auth_mode = 'WPA2PSK'
-                            key_name = 'WPAPSK'
-                        else:
-                            print 'Unknown AuthMode, can\'t complete ' + \
-                            'connection process!'
-                            return
-
-                        cmd_list = []
-                        cmd_list.append('NetworkType=' + info[6])
-                        cmd_list.append('AuthMode=' + auth_mode)
-                        cmd_list.append('EncrypType=' + info[4])
-                        cmd_list.append('SSID=' + info[2])
-                        cmd_list.append(key_name + '=' + network.get('key'))
-                        if info[5] == 'SHARED' and info[4] == 'WEP':
-                            cmd_list.append('DefaultKeyID=1')
-                        cmd_list.append('SSID=' + info[2])
-
-                        for cmd in cmd_list:
-                            cmd = 'iwpriv ' + self.iface + ' '
-                            if self.verbose: print cmd
-                            misc.Run(cmd)
+            if not (network.has_key('encryptype') and network.has_key('authmode')):
+                print 'Network dict is missing necessary keys. ' + \
+                      'Cannot authenticate. ' + str(network)
+                return
+            if network['encryptype'] == "WEP" and network['authtype'] == 'OPEN':
+                print 'Setting up WEP'
+                cmd = ''.join(['iwconfig ', self.iface, ' key ',
+                              network.get('key')])
+                if self.verbose: print cmd
+                misc.Run(cmd)
+            else:
+                cmd_list = []
+                cmd_list.append('NetworkType=' + network['nettype'])
+                cmd_list.append('AuthMode=' + network['authmode'])
+                cmd_list.append('EncrypType=' + network['encryptype'])
+                cmd_list.append('SSID="%s"' % network['essid'])
+                cmd_list.append('%s="%s"' % (network['keyname'], network['key']))
+                if network['nettype'] == 'SHARED' and network['encryptype'] == 'WEP':
+                    cmd_list.append('DefaultKeyID=1')
+    
+                for cmd in cmd_list:
+                    cmd = ['iwpriv', self.iface, 'set', cmd]
+                    if self.verbose: print cmd
+                    misc.Run(cmd)
 
     def GetBSSID(self, iwconfig=""):
         """ Get the MAC address for the interface. """
@@ -1102,7 +1151,7 @@ class WirelessInterface(Interface):
             output = misc.Run(cmd)
         else:
             output = iwconfig
-                
+
         bssid = misc.RunRegex(bssid_pattern, output)
         return bssid
 
@@ -1124,10 +1173,10 @@ class WirelessInterface(Interface):
         [(strength, max_strength)] = strength_pattern.findall(output)
         if max_strength and strength:
             return 100 * int(strength) // int(max_strength)
-        
+
         if strength is None:
             strength = misc.RunRegex(altstrength_pattern, output)
-        
+
         return strength
 
     def GetDBMStrength(self, iwconfig=None):
@@ -1166,4 +1215,3 @@ class WirelessInterface(Interface):
         if network:
             network = misc.to_unicode(network)
         return network
-
